﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Door2DoorApp
{
    //**********************************************************************************
    // ChangeCustomerForm.cs - This form of the application is used to switch the current
    // customer to some other customer that is currently in the Driver Application queue.
    // Opens and is referenced by the DriverApplication form which contains a listbox of
    // all of the current customer appointments.
    //**********************************************************************************
    public partial class ChangeCustomerForm : Form
    {
        DriverApp DriverForm;           //create a driver application instance
        public ChangeCustomerForm(ref DriverApp prevForm)
        {
            DriverForm = prevForm;      //set the driver app instance equal to the current driver application that is open
            InitializeComponent();
        }
        //event that occurs when this form loads
        private void ChangeCustomerForm_Load(object sender, EventArgs e)
        {
            //load all of the current customer appointments that are in the current driver app
            for (int i = 0; i < DriverForm.CustomerQueue.Items.Count; i++)
                CurrentCustomers.Items.Add(DriverForm.CustomerQueue.Items[i].ToString());
        }
        //events that occur when the cancel button on the form is clicked
        private void CancelBtn_Click(object sender, EventArgs e)
        {
            DriverForm.myTimer.Enabled = true;      //re-enable the timer on the driver application
            DriverForm.myTimer.Start();             //turn the timer on the driver application back on
            this.Close();                           //close this form instance
        }
        //events that occur when the ok button on the form is clicked
        private void OKBtn_Click(object sender, EventArgs e)
        {
            //if the user doesn't select any customer in the list, then prompt them to do so
            if (CurrentCustomers.SelectedIndex == -1)
                MessageBox.Show("You must select a customer appointment from the list above to make it the current one to drive.");
            //if they select the first customer in the list, then prompt them to select one who is not the current customer
            else if (CurrentCustomers.SelectedIndex == 0)
                MessageBox.Show("The customer you selected is already the current appointment being driven.");
            else
            {
                //if the first customer in the customer list is not the customer selected in this list box, then the user has a valid customer selection
                if (DriverForm.customerList[0].custName != DriverForm.customerList[CurrentCustomers.SelectedIndex].custName)
                {
                    //prompt the user if he/she wants to make the selected customer become the current customer appointment
                    DialogResult result = MessageBox.Show("Are you sure you would like to make customer " + DriverForm.customerList[CurrentCustomers.SelectedIndex].custName + " be your current appointment?", "Switch Customers?", MessageBoxButtons.YesNo);
                    //if the user selects yes, then do the following
                    if (result.ToString() == "Yes")
                    {
                        int custPos = CurrentCustomers.SelectedIndex;       //keep track of the customer's index in the listbox
                        Customers cust = DriverForm.customerList[0];        //set a customer instance equal to the current customer appointment
                        DriverForm.customerList[0] = DriverForm.customerList[custPos];      //set the first customer in the list equal to the customer currently currently selected
                        DriverForm.CustomerQueue.Items.RemoveAt(0);             //remove the first customer form the driver application's customer queue
                        //if the first customer in the list has been picked up, do the following
                        if (DriverForm.customerList[0].PickedUp == true)
                        {
                            //insert a customer to the top of the customer's queue in the driver application and say that he/she has been picked up
                            DriverForm.CustomerQueue.Items.Insert(0, DriverForm.customerList[custPos].custName + " - Been Picked Up");
                            //update the directions for the first customer to be picked up from the current location to their destination location
                            DriverForm.UpdateDirections(DriverForm.CurrentLocation, DriverForm.customerList[0].Destination.GetCoord());
                        }
                        else
                        {
                            //insert a customer to the top of the customer's queue in the driver application and say that he/she needs to be picked up
                            DriverForm.CustomerQueue.Items.Insert(0, DriverForm.customerList[custPos].custName + " - Needs to be Picked Up");
                            //update the directions for the first customer to be picked up from the current location to their home address
                            DriverForm.UpdateDirections(DriverForm.CurrentLocation, DriverForm.GetLatLngFromAdd(DriverForm.customerList[0].custAdd));
                        }
                        //set the customer at the current location equal to temporary customer that used to be the first customer in the list
                        DriverForm.customerList[custPos] = cust;
                        //remove the customer from the driver application's customer queue list box at the index that was selected in this form's listbox           
                        DriverForm.CustomerQueue.Items.RemoveAt(custPos);
                        //if the customer at the position selected in the list box has been picked up, do the following
                        if (DriverForm.customerList[custPos].PickedUp == true)
                            //insert a customer to the selected position index in this form's list box and say that he/she has been picked up
                            DriverForm.CustomerQueue.Items.Insert(custPos, cust.custName + " - Been Picked Up");
                        else
                            //insert a customer to the selected position index in this form's list box and say the he/she needs to be picked up
                            DriverForm.CustomerQueue.Items.Insert(custPos, cust.custName + " - Needs to be Picked Up");

                        DriverForm.myTimer.Enabled = true;      //re-enable the timer on the driver application
                        DriverForm.myTimer.Start();             //turn the timer on the driver application back on
                        this.Close();                           //close this form instance
                    }
                }
            }
        }
    }
}
