﻿using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Threading;
using System.Drawing;
using System.Net;
using System.Text;
using System.Timers;
using System.IO;
using System.Xml;
using System.Xml.XPath;
using System.Windows.Forms;
using GMap.NET;
using GMap.NET.WindowsForms;
using GMap.NET.MapProviders;
using GeoCoding;
using GeoCoding.Google;
using MySql.Data.MySqlClient;

namespace Door2DoorApp
{
    //**********************************************************************************
    // DriverApp.cs - The main application form where the driver can see the Google map of their
    // current location along with the markers of where they need to go. They can also get access
    // to the current customer appointments, as well as the distances, duration, and directions
    // for each of those appointments
    //**********************************************************************************
    public partial class DriverApp : Form
    {
        //Load all of the marker images that will be on the map into the application
        Bitmap marker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "marker.png"), true);
        Bitmap BlueMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "bluemarker.png"), true);
        Bitmap HospitalMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "hospital_icon.png"), true);
        Bitmap GolfMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "golf_icon.png"), true);
        Bitmap MovieMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "movie_icon.jpg"), true);
        Bitmap PostOfficeMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "post_office_icon.jpg"), true);
        Bitmap HQMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "taxi_icon.jpg"), true);
        Bitmap RestaurantMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "restaurant_icon.jpg"), true);
        Bitmap ChurchMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "church_icon.gif"), true);
        Bitmap DentistMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "dentist.png"), true);
        Bitmap GroceryStoreMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "grocery_store_icon.png"), true);
        Bitmap WalMartMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "walmart_icon.jpg"), true);
        Bitmap LibraryMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "library_icon.jpg"), true);
        Bitmap MuseumMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "museum_icon.jpg"), true);
        Bitmap RecCenterMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "rec_center_icon.png"), true);
        Bitmap VetMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "vet_icon.png"), true);
        Bitmap SalonMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "salon_icon.jpg"), true);
        Bitmap LaundromatMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "laundromat_icon.png"), true);
        Bitmap SeniorCenterMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "senior_center_icon.jpg"), true);
        Bitmap ApartmentsMarker = new Bitmap(System.IO.Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "apartment_icon.gif"), true);

        public int maxCust;             //variable containing the maximum amount of customers that the taxi can hold
        public int maxTime = 50000;     //variable setting the maximum amount of time that a new customer can appear on the queue, no greater than one
        public System.Timers.Timer myTimer = new System.Timers.Timer(50000);        //create a timer that will add a new customer to the queue whose max time is greater than one
        private BackgroundWorker worker;        //create a background worker instance to add a customer to the queue behind the scenes
        public bool AllCustsAdded = false;      //boolean variable to determine whether or not all of the customers have been added to the tool from the database or not
        public int CustomerCount = 0;           //variable to keep track of the customers that have been added from the database
        public int wheelchairCnt = 0;           //variable to keep track of all of the wheelchairs that are on the taxi
        public int TotalCust;                   //variable to keep track of the total customers that are in the database
        public int curCust = 0;                 //variable to keep track of the current customers that are in the customer list queue
        public string filepath = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
        //get the database's username, password, server and port from the global variables in the LoginForm class
        public string UserNm = Variables.UserName;
        public string Pass = Variables.Password;
        public string Server = Variables.Server;
        public string Port = Variables.PortNumber;
        UIForm InfoForm;                                //create a UIForm instance
  
        GMapOverlay overlayOne = new GMapOverlay();     //create a map overlay variable that will be used to place the current and destination marker locations on the map
        GMapOverlay LocationOverlay = new GMapOverlay();    //create a map overlay variable that will be used to place the 17 location markers on the map        
        List<Destinations> Places = new List<Destinations>();   //create a variable list of places from the 17 created in the class
        public List<Customers> customerList = new List<Customers>();    //create a variable list of customers
        public List<Customers> databaseCusts = new List<Customers>();   //create a variable list of customers that will come from the database
        public PointLatLng CurrentLocation = new PointLatLng();         //Latitude/Longitude variable to store what the current location is on the map

        //the driver app's public constructor containing a reference to the previous UIForm that was active before calling this form
        public DriverApp(ref UIForm prevForm)
        {
            InfoForm = prevForm;                //set the UIForm instance equal the previous UIForm that was active
            maxCust = InfoForm.GetMaxSeats;     //set the maximum number of customers equal to the value that was typed in by the user on the previous form
            SuspendLayout();                    //prebuilt function to allow the GMap control activate it's proper layout position on the form's screen
            InitializeComponent();
        }
        //when the form opens, perform the following tasks
        private void Form1_Load(object sender, EventArgs e)
        {
            this.WindowState = FormWindowState.Normal;      //set the form's window state to it's normal position as shown in design view
            Random rand = new Random();                     //create a new random variable
            myTimer.Interval = rand.Next(0, maxTime);       //set the time length interval equal to a random number between zero and the maximum time in the game
            myTimer.Enabled = true;                         //enable the timer so that it can begin running
            myTimer.Start();                                //start the timer
            worker = new BackgroundWorker();                //set the background worker instance equal to a new worker to add customers behind the scenes
            worker.DoWork += worker_DoWork;                 //increment the worker to the amount given in the worker_DoWork event function to be performed when the RunWorkerAsync() function is called
            myTimer.Elapsed += timer_Elapsed;               //increment the elapsed time of the timer equal to the amount given in the timer_Elapsed event function
        }
        //perform the following tasks when the GMap control loads
        private void gMapControl1_Load(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;            //set the current cursor image equal to the wait cursor
            //string used to connect to the MySQL server
            string MyConString = "SERVER= " + Server + ";PORT=" + Port + ";DATABASE=" + UserNm + ";UID=" + UserNm + ";PASSWORD=" + Pass;
            MySqlConnection conn = new MySqlConnection(MyConString);        //create a MySql connection equal to the connection given from the connection string above
            conn.Open();            //open that connection
            
            string query = "Select Count(*) From HartPlacesCoord";      //set the query to get the total records from the HartfordPlacesCoord table in the database
            MySqlCommand command = conn.CreateCommand();                //create a command for the MySql connection to run
            command.CommandText = query;                                //set the command's text value equal to the query above
            Int64 NumOfPlaces = Convert.ToInt64(command.ExecuteScalar());   //set the result of the query to a variable to store the total number of places

            //add all of the locations for places to go in Hartford
            for (int i = 1; i <= NumOfPlaces; i++)
            {
                Destinations newLocation = new Destinations();      //create a new destination class instance
                newLocation.ID = i;                                 //set the id of that destination location to the current value in the for loop
                //set the query to get the name of the location from the HartPlacesCoord table where the id is equal to the current value in the for loop
                query = "SELECT LocationName FROM HartPlacesCoord WHERE LocationID = '" + i + "';";
                command.CommandText = query;                        //set the command's text value equal to the query above
                newLocation.Name = Convert.ToString(command.ExecuteScalar());           //set the name of that destination location equal to the result of the query
                //set the query to get the latitude value of the location from the HartPlacesCoord table where the id is equal to the current value in the for loop
                query = "SELECT Latitude FROM HartPlacesCoord WHERE LocationID = '" + i + "';";
                command.CommandText = query;                        //set the command's text value equal to the query above
                newLocation.Latitude = Convert.ToDouble(command.ExecuteScalar());       //set the latitude value of that destination equal to the result of the query
                //set the query to get the longitude value of the location from the HartPlacesCoord table where the id is equal to the current value in the for loop
                query = "SELECT Longitude FROM HartPlacesCoord WHERE LocationID = '" + i + "';";
                command.CommandText = query;                        //set the command's text value equal to the query above
                newLocation.Longitude = Convert.ToDouble(command.ExecuteScalar());      //set the longitude value of that destination equal to the result of the query
                Places.Add(newLocation);                            //add this destination location to the Places list
            }

            query = "Select Count(*) From CustomersTbl";         //set the query to get the total records from the CustomersTbl table in the database
            command.CommandText = query;                        //set the command's text value equal to the query above
            TotalCust = Convert.ToInt16(command.ExecuteScalar());       //set the total number of customers equal to the result of the query
            //if the total customers in the database is greater than the amount that the taxi can hold, set the total customers equal to the maximum taxi size
            if (TotalCust > maxCust)
                TotalCust = maxCust;
            //if there are no customers in the queue currently, do the following
            if (CustomerQueue.Text == "")
            {
                //for every customer in the database
                for (int i = 0; i < TotalCust; i++)
                {
                    Customers CurCust = new Customers();        //create a new customer class instance
                    CurCust.Destination = new Destinations();       //set the destination for that customer equal to a new destination class instance
                    //set the query to get the customer name from the CustomersTbl where the customer id is equal to the current value of the for loop
                    query = "SELECT CustName FROM CustomersTbl WHERE CustID = '" + i + "';";
                    command.CommandText = query;                //set the command's text value equal to the query above
                    CurCust.custName = Convert.ToString(command.ExecuteScalar());       //set the name of the customer equal to the result of the query
                    //set the query to get the address of the customer from the CustomersTbl where the customer id is equal to the current value of the for loop
                    query = "SELECT Address FROM CustomersTbl WHERE CustID = '" + i + "';";
                    command.CommandText = query;                //set the command's text value equal to the query above
                    CurCust.custAdd = Convert.ToString(command.ExecuteScalar());    //set the address of the customer equal to the result of the query
                    //set the query to get the customer's desired destination from the CustomersTbl where the customer id is equal to the current value of the for loop
                    query = "SELECT Destination FROM CustomersTbl WHERE CustID = '" + i + "';";
                    command.CommandText = query;                //set the command's text value equal to the query above
                    CurCust.Destination.Name = Convert.ToString(command.ExecuteScalar());   //set the destination of the customer equal to the result of the query
                    //set the query to get the latitude coordinate of the customer's desired destination from the HartPlacesCoord database table
                    query = "Select Latitude FROM HartPlacesCoord WHERE LocationName = '" + CurCust.Destination.Name + "';";
                    command.CommandText = query;                //set the command's text value equal to the query above
                    //set the latitude coordinate of the customer's desired destination to the result of the query
                    CurCust.Destination.Latitude = Convert.ToDouble(command.ExecuteScalar());   
                    //set the query to get the longitude coordinate of the customer's desired destination from the HartPlacesCoord database table
                    query = "Select Longitude FROM HartPlacesCoord WHERE LocationName = '" + CurCust.Destination.Name + "';";
                    command.CommandText = query;                //set the command's text value equal to the query above
                    CurCust.Destination.Longitude = Convert.ToDouble(command.ExecuteScalar());
                    //set the query to determine whether the customer at the for loop's location value is on the bus or not
                    query = "Select OnBus FROM CustomersTbl WHERE CustID = '" + i + "';";
                    command.CommandText = query;                //set the command's text value equal to the query above
                    //if the value from the query is equal to 1, set the picked up value for that customer equal to true
                    if (Convert.ToString(command.ExecuteScalar()) == "1")
                        CurCust.PickedUp = true;
                    else
                        CurCust.PickedUp = false;       //otherwise set the picked up value to false for that customer
                    //set the query to determine whether the customer at the for loop's location value has a wheelchair or not
                    query = "SELECT HasWheelchair FROM CustomersTbl WHERE CustID = '" + i + "';";
                    command.CommandText = query;            //set the command's text value equal to the query above
                    //if the value from the query is equal to 1, set the has wheelchair value to true
                    if (Convert.ToString(command.ExecuteScalar()) == "1")
                        CurCust.HasWheelChair = true;
                    else
                        CurCust.HasWheelChair = false;      //otherwise set the has wheelchair value to false for that customer
                    customerList.Add(CurCust);              //add that customer and its attributes to the the customer list
                    databaseCusts.Add(CurCust);
                }
                //add the first customer in the list to the top of the queue
                CustomerQueue.Items.Add(customerList[0].custName + " - Needs to be Picked Up");
            }
            MaxPassBox.Text = maxCust.ToString();           //set the maximum passengers value on the form to the maximum customer value
            TotalAppts.Text = CustomerQueue.Items.Count.ToString();

            // configure the map by docking the map, using the Google map provider, and editing the zoom settings
            this.gMapControl1.Dock = DockStyle.Fill;
            this.gMapControl1.Manager.Mode = AccessMode.ServerAndCache;
            this.gMapControl1.MapProvider = GMapProviders.GoogleMap;
            this.gMapControl1.MinZoom = 1;
            this.gMapControl1.MaxZoom = 20;
            this.gMapControl1.Zoom = 15;

            //add all of the location markers on the map
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[0].GetCoord(), HQMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[1].GetCoord(), RestaurantMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[2].GetCoord(), HospitalMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[3].GetCoord(), WalMartMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[4].GetCoord(), ChurchMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[5].GetCoord(), LibraryMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[6].GetCoord(), MuseumMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[7].GetCoord(), PostOfficeMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[8].GetCoord(), RecCenterMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[9].GetCoord(), VetMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[10].GetCoord(), MovieMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[11].GetCoord(), DentistMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[12].GetCoord(), GroceryStoreMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[13].GetCoord(), SalonMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[14].GetCoord(), GolfMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[15].GetCoord(), LaundromatMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[16].GetCoord(), SeniorCenterMarker));
            overlayOne.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(Places[17].GetCoord(), ApartmentsMarker));
            
            //create new lat/long starting and ending location instances
            PointLatLng StartLocation = new PointLatLng();
            PointLatLng EndLocation = new PointLatLng();
            //if the HQ button was selected on the previous UIForm, then start the map location at HQ
            if (InfoForm.HQStart == true)
            {
                this.gMapControl1.Position = Places[0].GetCoord();
                StartLocation = Places[0].GetCoord();
            }
            //otherwise if the user selected latitude and longitude coordinates from the UIForm, start the map at that coordinate location
            else if (InfoForm.GetStartLatitude != "" && InfoForm.GetStartLongitude != "")
            {
                StartLocation.Lat = Convert.ToDouble(InfoForm.GetStartLatitude);
                StartLocation.Lng = Convert.ToDouble(InfoForm.GetStartLongitude);
                this.gMapControl1.Position = new PointLatLng(StartLocation.Lat, StartLocation.Lng);
            }
            //otherwise if the user typed in an address from the UIForm, start the map at that address
            else
            {
                this.gMapControl1.SetCurrentPositionByKeywords(InfoForm.GetStartAddress);
                StartLocation = GetLatLngFromAdd(InfoForm.GetStartAddress);
            }
            CurrentLocation = StartLocation;            //set the current location of the taxi equal to the starting address
            EndLocation = GetLatLngFromAdd(customerList[0].custAdd);        //set the ending address equal to the first customer's address
            UpdateDirections(CurrentLocation, EndLocation);             //update the directions from the starting location, to the current customer's address

            this.gMapControl1.Overlays.Add(overlayOne);         //add the destination locations' overlays on the map control
            this.gMapControl1.Overlays.Add(LocationOverlay);    //add the overlay to display all of the locations markers on the map control
            Cursor.Current = Cursors.Default;                   //reset the cursor to the normal cursor for the application
        }
        //events that occur when the close button is clicked
        private void CloseBtn_Click(object sender, EventArgs e)
        {
            myTimer.Enabled = false;
            myTimer.Stop();
            //prompt the user if they want to exit the application or not, if they do close the environment, otherwise keep the app open
            DialogResult result1 = MessageBox.Show("Are you sure you would like to exit the application?", "Quit Application", MessageBoxButtons.YesNo);
            if (result1.ToString() == "Yes")
                Environment.Exit(0);
            else
            {
                myTimer.Enabled = true;
                myTimer.Start();
            }
        }
        //events that occur when the user clicks the x button in the top right hand corner of the screen
        private void DriverApp_FormClosing(object sender, FormClosingEventArgs e)
        {
            //if the current form is still active
            if (this.Visible == true)
            {
                myTimer.Enabled = false;            //temporarily disable the game timer
                myTimer.Stop();                     //temporarily stop the game timer
                //prompt the user if they want to exit the application or not, if they do close the environment, otherwise keep the app open
                DialogResult result1 = MessageBox.Show("Are you sure you would like to exit the application?", "Quit Application", MessageBoxButtons.YesNo);
                if (result1.ToString() == "Yes")
                    Environment.Exit(0);
                else
                {
                    e.Cancel = false;               //don't allow the form to close execution
                    myTimer.Enabled = true;         //re-enable the game timer
                    myTimer.Start();                //re-start the game timer
                }
            }
        }

        //events that occur when the user clicks the cancel appointment button
        private void CancelApptBtn_Click(object sender, EventArgs e)
        {
            myTimer.Enabled = false;            //temporarily disable the game timer
            myTimer.Stop();                     //temporarily stop the game timer
            //if the user didn't select any customer in the list, prompt them
            if (CustomerQueue.SelectedIndex == -1)
                MessageBox.Show("You must select a customer appointment from the list above before cancelling.");
            //if there are no customers in the list currently, prompt them
            else if (CustomerQueue.Items.Count < 1)
                MessageBox.Show("There are no customers on the bus currently, so no appointments can be cancelled at this time.");
            //if there is only one customer in the list currently
            else if (CustomerQueue.Items.Count == 1)
            {
                //prompt them if they want to cancel the last customer appointment available in the queue
                DialogResult result = MessageBox.Show("Are you sure you would like to cancel the selected appointment for customer " + customerList[0].custName + "?", "Cancel Appointment", MessageBoxButtons.YesNo);
                //if the user selects yes
                if (result.ToString() == "Yes")
                {
                    //if the first customer in the list has been picked up already
                    if (customerList[0].PickedUp == true)
                    {
                        curCust--;                  //decrement the number of current customers on the taxi
                        //if the first customer in the list has a wheelchair
                        if (customerList[0].HasWheelChair == true)
                            wheelchairCnt--;        //decrement the amount of wheelchairs on the bus
                    }
                    //remove all markers from the location overlay on the screen
                    LocationOverlay.Markers.RemoveAt(0);
                    LocationOverlay.Markers.RemoveAt(0);
                    //add the red marker on top of the blue marker to show only the current location on the map
                    LocationOverlay.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(CurrentLocation, BlueMarker));
                    LocationOverlay.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(CurrentLocation, marker));
                    MilesLabel.Text = "0";                          //set the mileage label to 0
                    Duration.Text = "0";                            //set the duration label to 0
                    Directions.Text = "";                           //clear all directions from the application
                    FromAdd.Text = "";                              //clear the from address from the screen
                    ToAdd.Text = "";                                //clear the to address from the screen
                    CustomerQueue.Items.RemoveAt(0);    //remove the first customer from the application's customer queue
                    customerList.RemoveAt(0);           //remove the first customer from the customer list
                    WheelchairBox.Text = wheelchairCnt.ToString();      //display the current wheelchair count
                    PassCntBox.Text = (curCust + wheelchairCnt).ToString();     //display the current customer count
                    TotalAppts.Text = CustomerQueue.Items.Count.ToString();     //display the total appointments currently in the queue
                }
            }
            //if there is more than one customer in the list currently
            else
            {
                //prompt them if they want to cancel the current customer appointment in the queue
                DialogResult result = MessageBox.Show("Are you sure you would like to cancel the selected appointments?", "Cancel Appointments", MessageBoxButtons.YesNo);
                //if the user selects yes
                if (result.ToString() == "Yes")
                {
                    //for every customer currently in the queue
                    for (int i = 0; i < CustomerQueue.Items.Count; i++)
                    {
                        //for every customer that is selected in the queue
                        for (int j = 0; j < CustomerQueue.SelectedItems.Count; j++)
                        {
                            //if the customer selected is equal to the current customer being checked in the queue, we have the correct customer to remove
                            if (CustomerQueue.Items[i].ToString() == CustomerQueue.SelectedItems[j].ToString())
                            {
                                //if this customer has been picked up
                                if (customerList[i].PickedUp == true)
                                {
                                    curCust--;                  //decrement the number of customers on the taxi
                                    //if this customer has a wheelchair
                                    if (customerList[i].HasWheelChair == true)
                                        wheelchairCnt--;        //decrement the wheelchair count
                                }
                                CustomerQueue.Items.RemoveAt(i);    //remove that customer from the queue
                                customerList.RemoveAt(i);           //remove that customer from the customer list
                                j--;                        //decrement the amount of selected items in the queue
                            }
                        }
                    }
                    SelectedCustomers.Text = "";                //reset the amount of selected customers to nothing
                    WheelchairBox.Text = wheelchairCnt.ToString();      //display the amount of wheelchairs on the taxi
                    PassCntBox.Text = (curCust + wheelchairCnt).ToString();     //display the amount of current customers are on the taxi
                    TotalAppts.Text = CustomerQueue.Items.Count.ToString();     //display the total number of customer appointments read in from the database
                    //if there is at least one customer on the taxi
                    if (curCust > 0)
                    {
                        //if the first customer in the list has been picked up
                        if (customerList[0].PickedUp == true)
                            //update the current location directions to begin at the current location and go to the customer's destination address
                            UpdateDirections(CurrentLocation, customerList[0].Destination.GetCoord());
                        //if the customer has not been picked up yet
                        else
                            //update the current location directions to begin at the current location and go to the customer's home address
                            UpdateDirections(CurrentLocation, GetLatLngFromAdd(customerList[0].custAdd));
                    }
                    //if there is only one customer on the taxi
                    else
                    {
                        MilesLabel.Text = "0";          //reset the current destination's mileage to 0
                        Duration.Text = "0";            //reset the current destination's duration to 0
                        FromAdd.Text = "";              //reset the current destination's from address to null
                        ToAdd.Text = "";                //reset the current destination's to address to null
                        Directions.Text = "";           //reset the destination's directions to null
                    }
                    SelectedCustomers.Items.Clear();    //clear all items from the selected customers list box
                    SelectedAdd.Text = "";              //reset the selected customer's address to null
                    SelectedDestination.Text = "";      //reset the selected customer's destination address to null
                    curMilesCnt.Text = "0";             //reset the selected customer's mileage to 0
                    curStepsCnt.Text = "0";             //reset the selected customer's number of directional steps to 0
                    curTimeCnt.Text = "0";              //reset the selected customer's duration time to 0
                    TotalMiles.Text = "0";              //reset the selected customers' total mileage to 0
                    TotalTime.Text = "0";               //reset the selected customers' total duration time to 0
                    StepsCnt.Text = "0";                //reset the selected customers' directional steps to 0
                }
            }
            myTimer.Enabled = true;             //re-enable the game timer
            myTimer.Start();                     //re-start the game timer
        }

        //events that take place when the arrived button is clicked
        private void ArrivedBtn_Click(object sender, EventArgs e)
        {
            myTimer.Enabled = false;        //temporarily disable the game timer
            myTimer.Stop();                 //temporarily stop the game timer
            //if there are not customers in the taxi, prompt them
            if (CustomerQueue.Items.Count < 1)
                MessageBox.Show("There are no customers on the bus currently to take to their destination.");
            //otherwise if there is only one customer currently in the queue
            else if (CustomerQueue.Items.Count == 1)
            {
                //if this customer has been picked up already
                if (customerList[0].PickedUp == true)
                {
                    //set the current location to this customer's destination
                    CurrentLocation = customerList[0].Destination.GetCoord();
                    //prompt them that they have arrived at the customer's destination
                    MessageBox.Show("You've arrived at your current destination for " + customerList[0].custName + ".");
                    //update the directions to be at the customer's current location
                    UpdateDirections(CurrentLocation, CurrentLocation);
                    customerList.RemoveAt(0);           //remove the current customer from the customer list
                    CustomerQueue.Items.RemoveAt(0);    //remove the current customer from the form's customer queue
                    curCust--;                          //decrement the number of current customers on the taxi
                    //if this customer has a wheelchair
                    if (customerList[0].HasWheelChair == true)
                        wheelchairCnt--;                //decrement the number of wheelchairs
                }
                //if this customer has not yet been picked up
                else
                {
                    //set the current location to the current customer's address
                    CurrentLocation = GetLatLngFromAdd(customerList[0].custAdd);
                    //alert the user that they've picked up that customer
                    MessageBox.Show("You've picked up " + customerList[0].custName + ".");
                    //make the first customer in the queue equal to this customer marking that he's been picked up
                    CustomerQueue.Items[0] = customerList[0].custName + " - Been Picked Up";
                    //update the directions to be from the current location to this customer's destination location
                    UpdateDirections(CurrentLocation, customerList[0].Destination.GetCoord());
                    //if the first customer has a wheelchair
                    if (customerList[0].HasWheelChair == true)
                        wheelchairCnt++;                    //increment the number of wheelchairs on the taxi
                    curCust++;                              //increment the number of customers on the taxi
                    customerList[0].PickedUp = true;            //mark that this customer has been picked up
                }
                TotalAppts.Text = CustomerQueue.Items.Count.ToString();     //display the total appointments currently available in the queue
                WheelchairBox.Text = wheelchairCnt.ToString();              //display the amount of wheelchairs on the taxi
                PassCntBox.Text = (curCust + wheelchairCnt).ToString();     //display the amount of customers on the taxi
            }
            //if there are more than one customers on the taxi
            else
            {
                //if the first customer has been picked up
                if (customerList[0].PickedUp == true)
                {
                    CurrentLocation = customerList[0].Destination.GetCoord();       //set the current location equal to the first customer's destination location
                    //if the second customer in the queue has been picked up
                    if (customerList[1].PickedUp == true)
                    {
                        //alert them that they've arrived at the first customer's current location and they are to pick up the next customer in the list
                        MessageBox.Show("You've arrived at your current destination for " + customerList[0].custName + ". Next stop is for " + customerList[1].custName + ".");
                        //update the directions from the current taxi location to the next customer's current destination
                        UpdateDirections(CurrentLocation, customerList[1].Destination.GetCoord());
                    }
                    //if the second customer in the queue needs to be picked up
                    else
                    {
                        //alert them that they've arrived at the first customer's current location and they are to pick up the next customer in the list
                        MessageBox.Show("You've arrived at your current destination for " + customerList[0].custName + ". Next stop is to pick up " + customerList[1].custName + ".");
                        //update the directions from the current taxi location to the next customer's home address
                        UpdateDirections(CurrentLocation, GetLatLngFromAdd(customerList[1].custAdd));
                    }
                    //if the first customer has a wheelchair
                    if (customerList[0].HasWheelChair == true)
                        wheelchairCnt--;                //decrement the wheelchair count by 1
                    curCust--;                              //decrement the number of current customers on the taxi
                    customerList.RemoveAt(0);               //remove the first customer from the list
                    CustomerQueue.Items.RemoveAt(0);        //remove the first customer from the form's customer queue
                }
                //if the first customer needs to be picked up
                else
                {
                    //if the number of customers and wheelchairs on the bus are greater than or equal to the maximum customers that can be on the bus
                    if (curCust + wheelchairCnt >= maxCust)
                        //prompt them that there isn't enough room on the bus to pick up another customer and that a customer needs to be dropped off to make room
                        MessageBox.Show("You cannot pick up customer " + customerList[0].custName + " because there are no more available seats on the bus. Drop off a customer before picking another one up.");
                    //if there is enough room on the taxi to pick up another customer
                    else
                    {
                        CurrentLocation = GetLatLngFromAdd(customerList[0].custAdd);                //set the current location equal to the first customer's address
                        MessageBox.Show("You've picked up " + customerList[0].custName);            //alert the user that they've picked up the first customer
                        CustomerQueue.Items[0] = customerList[0].custName + " - Been Picked Up";    //set the first customer in the queue to the current customer and mark that they've been picked up
                        //if the first customer has a wheelchair
                        if (customerList[0].HasWheelChair == true)
                            wheelchairCnt++;        //increment the number of customers on the taxi
                        curCust++;                  //increment the number of customers on the taxi
                        customerList[0].PickedUp = true;                            //mark that the first customer in the queue has been picked up
                        //update the directions to be from the current location to the first customer's destination location
                        UpdateDirections(CurrentLocation, customerList[0].Destination.GetCoord());
                    }
                }
                TotalAppts.Text = CustomerQueue.Items.Count.ToString();     //set the total appointments value equal to the number of appointments in the customer queue
                WheelchairBox.Text = wheelchairCnt.ToString();              //set the wheelchair value equal to the current wheelchairs on the bus
                PassCntBox.Text = (curCust + wheelchairCnt).ToString();     //set the customers value to the current customers on the bus
                //if the customer count and wheelchair count is equal to the maximum customers that can fit on the bus, 
                //prompt them that it is full and that they need to start dropping off customers to make room
                if (curCust + wheelchairCnt == maxCust)
                    MessageBox.Show("You have reached the maximum number of riders on this bus. You must drop someone off at a destination before you pick anyone else up.");
            }
            myTimer.Enabled = true;         //re-enable the game timer
            myTimer.Start();                //re-start the game timer
        }
        //events that take place when the change customer button is clicked
        private void ChangeCustBtn_Click(object sender, EventArgs e)
        {
            //if there are not customers in the queue, prompt them
            if (CustomerQueue.Items.Count < 1)
                MessageBox.Show("There are no current customers on the bus available to change appointments with.");
            //if there is only one customer in the queue, prompt them
            else if (CustomerQueue.Items.Count == 1)
                MessageBox.Show("There is only one customer currently in the queue, so no appointments can be swapped.");
            //if there is more than one customer in the queue
            else
            {
                myTimer.Enabled = false;            //pause the timer for now
                myTimer.Stop();                     //temporarily stop the game timer
                DriverApp DriverForm = this;        //create a driver application instance and set it to this driver application form
                //create a change form instance referencing this driver application
                ChangeCustomerForm ChangeForm = new ChangeCustomerForm(ref DriverForm);
                ChangeForm.Show();          //reveal this change customer form
            }
        }

        //Zoom In
        private void pictureBox5_Click(object sender, EventArgs e)
        {
            this.gMapControl1.Zoom += 1;        //increment the zoom value by 1
            this.gMapControl1.Refresh();        //refresh the map
        }
        //Zoom Out
        private void pictureBox6_Click(object sender, EventArgs e)
        {
            this.gMapControl1.Zoom -= 1;        //decrement the zoom value by 1
            this.gMapControl1.Refresh();        //refresh the map
        }
        //up arrow
        private void pictureBox4_Click(object sender, EventArgs e)
        {
            double ZoomOffset = GetPanOffsetAmt();              //call a function to determine what to set the zoom offset value to
            double Latitude = this.gMapControl1.Position.Lat + ZoomOffset;      //increase the latitude value by the zoom offset
            double Longitude = this.gMapControl1.Position.Lng;                  //keep the longitude value equal to the current map's longitude
            this.gMapControl1.Position = new PointLatLng(Latitude, Longitude);  //set the center position of the map to this new lat/long position
            this.gMapControl1.Update();             //update the map
        }
        //down arrow
        private void pictureBox1_Click(object sender, EventArgs e)
        {
            double ZoomOffset = GetPanOffsetAmt();              //call a function to determine what to set the zoom offset value to
            double Latitude = this.gMapControl1.Position.Lat - ZoomOffset;     //decrease the latitude value by the zoom offset
            double Longitude = this.gMapControl1.Position.Lng;                  //keep the longitude value equal to the current map's longitude
            this.gMapControl1.Position = new PointLatLng(Latitude, Longitude);  //set the center position of the map to this new lat/long position
            this.gMapControl1.Update();             //update the map
        }
        //right arrow
        private void pictureBox3_Click(object sender, EventArgs e)
        {
            double ZoomOffset = GetPanOffsetAmt();             //call a function to determine what to set the zoom offset value to
            double Latitude = this.gMapControl1.Position.Lat;       //keep the latitude value equal to the current map's latitude
            double Longitude = this.gMapControl1.Position.Lng + ZoomOffset;     //increase the longitude value by the zoom offset amount
            this.gMapControl1.Position = new PointLatLng(Latitude, Longitude);  //set the center position of the map to this new lat/long position
            this.gMapControl1.Update();             //update the map
        }
        //left arrow
        private void pictureBox2_Click(object sender, EventArgs e)
        {
            double ZoomOffset = GetPanOffsetAmt();              //call a function to determine what to set the zoom offset value to
            double Latitude = this.gMapControl1.Position.Lat;       //keep the latitude value equal to the current map's latitude
            double Longitude = this.gMapControl1.Position.Lng - ZoomOffset;     //decrease the longitude value by the zoom offset amount
            this.gMapControl1.Position = new PointLatLng(Latitude, Longitude);      //set the center position of the map to this new lat/long position
            this.gMapControl1.Update();             //update the map
        }
        //**************************************************************************************
        // GetLatLngFromAddress function: Function that will take an address string and return
        // a latitude/longitude coordinate location of that address using a google map api
        // that will connect to the internet and return a coordinate from the Google Maps server
        //**************************************************************************************
        public PointLatLng GetLatLngFromAdd(string Address)
        {
            PointLatLng returnPos = new PointLatLng();          //create a new latitude/longitude position
            //set a url equal to the google map's api with an xml geocode containing the address called from the function
            string URL = "http://maps.googleapis.com/maps/api/geocode/" + "xml?address=" + Address + ",+CA&sensor=false";
            WebResponse response = null;            //create a webresponse instance
            //test the address's web request to see if the address can be processed
            try
            {
                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(URL);        //create a new http web request with the url given above
                request.Method = "GET";             //set the request method to get the http web request
                response = request.GetResponse();           //set the response from the request given
                //if the response is not blank
                if (response != null)
                {
                    //create a new XPathDocument instance with the document to receive the response stream
                    XPathDocument document = new XPathDocument(response.GetResponseStream());
                    XPathNavigator navigator = document.CreateNavigator();      //create a new XPathNavigator instance with the document that creates the navigator

                    // get response status code
                    XPathNodeIterator statusIterator = navigator.Select("/GeocodeResponse/status");
                    // while not at the end of the status iterator value
                    while (statusIterator.MoveNext())
                    {
                        //if the current value of the iterator is not valid
                        if (statusIterator.Current.Value != "OK")
                        {
                            //respond with a status for that status error with it's current value
                            MessageBox.Show("Error: response status = '" + statusIterator.Current.Value + "'");
                            //return a -1 for the latitude and longitude coordinate locations for the map
                            returnPos.Lat = -1;
                            returnPos.Lng = -1;
                            return returnPos;
                        }
                    }
                    // get results with an XPathNodeIterator using the Geocode result response
                    XPathNodeIterator resultIterator = navigator.Select("/GeocodeResponse/result");
                    // while not at the end of the result iterator value
                    while (resultIterator.MoveNext())
                    {
                        //create a new XPathNodeIterator instance with the geometry selected
                        XPathNodeIterator geometryIterator = resultIterator.Current.Select("geometry");
                        //while not at the end of the geometry iterator value
                        while (geometryIterator.MoveNext())
                        {
                            //create a new XPathNodeIterator instance with the location selected
                            XPathNodeIterator locationIterator = geometryIterator.Current.Select("location");
                            //while not at the end of the location iterator value
                            while (locationIterator.MoveNext())
                            {
                                //create a new XPathNodeIterator instance with the latitude selected
                                XPathNodeIterator latIterator = locationIterator.Current.Select("lat");
                                //while not at the end of the latitude iterator value
                                while (latIterator.MoveNext())
                                    //set the latitude of the postition to be returned to the iterator value
                                    returnPos.Lat = Convert.ToDouble(latIterator.Current.Value);
                                //create a new XPathNodeIterator instance with the longitude selected
                                XPathNodeIterator lngIterator = locationIterator.Current.Select("lng");
                                //while not at the end of the longitude iterator value
                                while (lngIterator.MoveNext())
                                    //set the longitude of the position to be returned to the iterator value
                                    returnPos.Lng = Convert.ToDouble(lngIterator.Current.Value);
                            }
                        }
                    }
                }
            }
            //if an exception was caught by the web address given display the message of the exception
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
            //after the request has been processed or if an exception was caught
            finally
            {
                //if the response is not null
                if (response != null)
                {
                    response.Close();       //remove the response
                    response = null;        //and set the response to null
                }
            }
            return returnPos;               //return the resulted position from the iterator
        }

        //*******************************************************************************
        // UpdateDirections function: Function given the starting and ending location which 
        // will display the mileage, duration, and direction information from those locations
        // on the map.
        //*******************************************************************************
        public void UpdateDirections(PointLatLng StartLocation, PointLatLng EndLocation)
        {
            Directions.Clear();         //clear all of the current directions being displayed
            GDirections dir = new GDirections();        //create a new directions instance
            //create a Google Map directions variable instance with the starting and ending location result
            var x = GMapProviders.GoogleMap.GetDirections(out dir, StartLocation, EndLocation, false, false, false, true, false);
            //if the directions found were valid
            if (x == DirectionsStatusCode.OK)
            {
                string OutputString = "";           //create an output string to null
                int DirCnt = 1;                     //set the count of the directions to the first direction
                dir.Steps = dir.Steps.Distinct().ToList();      //set the number of steps for that direction instance to the distinct number from the list
                FromAdd.Text = dir.StartAddress;                //set the from address value to the directions start address
                ToAdd.Text = dir.EndAddress;                    //set the to address value to the directions end address
                MilesLabel.Text = dir.Distance.ToString();      //set the miles value to the directions distance from the two locations
                Duration.Text = dir.Duration.ToString();        //set the duration value to the duration time from the two locations
                //for every step in the direction's steps
                foreach (var steps in dir.Steps)
                {
                    //set the output value equal to the direction number and the directions for that step (with the removal of the html value for that step)
                    OutputString = DirCnt + ".) " + HtmlRemoval.StripTagsCharArray(steps.HtmlInstructions);
                    Directions.Text = Directions.Text + OutputString + Environment.NewLine;         //add the current direction to the list of the total directions
                    DirCnt++;               //increment the number of directions for the location
                }
                //if the number of the markers for the locations on the map is greater than 0
                if (LocationOverlay.Markers.Count > 0)
                {        
                    //remove the starting and ending marker from the location overlay
                    LocationOverlay.Markers.RemoveAt(0);
                    LocationOverlay.Markers.RemoveAt(0);
                }
                //add the starting and ending location on the map
                LocationOverlay.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(StartLocation, marker));
                LocationOverlay.Markers.Add(new GMap.NET.WindowsForms.Markers.GMarkerGoogle(EndLocation, BlueMarker));
                gMapControl1.Position = CurrentLocation;        //set the current center position location to the current location
            }
        }
        //********************************************************************************
        // GetPanOffsetAmt function: Function that will return the panning value on the map
        // based on what setting the zoom is on for the map control. Size determined for each
        // setting based on testing values
        //********************************************************************************
        public double GetPanOffsetAmt()
        {
            if (this.gMapControl1.Zoom == 1)
                return 50;
            else if (this.gMapControl1.Zoom == 2)
                return 25;
            else if (this.gMapControl1.Zoom == 3)
                return 15;
            else if (this.gMapControl1.Zoom == 4)
                return 7.5;
            else if (this.gMapControl1.Zoom == 5)
                return 3.75;
            else if (this.gMapControl1.Zoom == 6)
                return 1.875;
            else if (this.gMapControl1.Zoom == 7)
                return 1;
            else if (this.gMapControl1.Zoom == 8)
                return 0.5;
            else if (this.gMapControl1.Zoom == 9)
                return 0.25;
            else if (this.gMapControl1.Zoom == 10)
                return 0.125;
            else if (this.gMapControl1.Zoom == 11)
                return 0.0625;
            else if (this.gMapControl1.Zoom == 12)
                return 0.04;
            else if (this.gMapControl1.Zoom == 13)
                return 0.02;
            else if (this.gMapControl1.Zoom == 14)
                return 0.01;
            else if (this.gMapControl1.Zoom == 15)
                return 0.005;
            else if (this.gMapControl1.Zoom == 16)
                return 0.0025;
            else if (this.gMapControl1.Zoom == 17)
                return 0.001;
            else if (this.gMapControl1.Zoom == 18)
                return 0.0005;
            else if (this.gMapControl1.Zoom == 19)
                return 0.00025;
            else
                return 0.0001;
        }
        //event that will take place when a value in the queue is changed from the current customer value
        private void CustomerQueue_SelectedIndexChanged(object sender, EventArgs e)
        {
            int CustTime = 0;           //variable to keep track of the duration time for the selected customer's appointment
            double CustMiles = 0;       //variable to keep track of the mileage for the selected customer's appointment
            int CustSteps = 0;          //variable to keep track of the steps required for the selected customer's appointment
            Customers cust = new Customers();       //create a new customer instance
            GDirections dir = new GDirections();    //create a new directions instance
            SelectedCustomers.Items.Clear();        //clear all of the customers from the selected customers queue
            //create a list of strings that will contain the names of all of the customers that were selected in the customer queue
            List<string> selectedCusts = new List<string>();
            //for every customer selected in the customer queue
            for (int j = 0; j < CustomerQueue.SelectedItems.Count; j++)
                selectedCusts.Add(CustomerQueue.SelectedItems[j].ToString());       //add the selected customer to the queue list
            //for every customer in the queue
            for (int i = 0; i < CustomerQueue.Items.Count; i++)
            {
                //for every customer in the selected customers list
                for (int j = 0; j < selectedCusts.Count; j++)
                {
                    //if the current customer in the queue is equal to the current customer in the selected customers list
                    if (CustomerQueue.Items[i].ToString() == selectedCusts[j])
                    {
                        cust = customerList[i];             //set the customer instance equal to current customer in the for loop
                        PointLatLng StartLocation = GetLatLngFromAdd(cust.custAdd);     //set the start location to the customer's home address
                        PointLatLng EndLocation = cust.Destination.GetCoord();          //set the end location to the customer's destination location
                        //create a Google Map directions variable instance with the starting and ending location result
                        var x = GMapProviders.GoogleMap.GetDirections(out dir, StartLocation, EndLocation, false, false, false, true, false);
                        //if the directions found were valid
                        if (x == DirectionsStatusCode.OK)
                        {
                            SelectedCustomers.Items.Add(cust.custName);         //add this customer to the selected customers queue
                            //set the total customer time equal to the current customer time plus the total customer time of all the customers
                            CustTime = CustTime + Convert.ToInt16(dir.Duration.Substring(0, 2));
                            //set the total customer mileage equal to the current customer miles plus the total customer mileage of all the customers
                            CustMiles = CustMiles + Convert.ToDouble(dir.Distance.Substring(0, 3));
                            //set the total steps for the customers equal to the current customer steps plust the total customer steps for all the customers
                            CustSteps = CustSteps + dir.Steps.Count;
                        }
                        //set the total time value equal to the customer time value
                        TotalTime.Text = CustTime.ToString() + " min";
                        //set the total mileage value equal to the customer mileage value
                        TotalMiles.Text = CustMiles.ToString() + " miles";
                        //set the total steps equal to the total customer steps value
                        StepsCnt.Text = CustSteps.ToString() + " steps";
                    }
                }
            }
        }
        //event that occurs when the value of the selected customers queue changes
        private void SelectedCustomers_SelectedIndexChanged(object sender, EventArgs e)
        {
            //if there are no customers currently selected
            if (SelectedCustomers.SelectedItems.Count == 0)
            {
                SelectedDestination.Text = "";      //reset the selected destination value to null
                SelectedAdd.Text = "";              //reset the selected address value to null
                curMilesCnt.Text = "0";             //reset the current customer's mileage value to 0
                curStepsCnt.Text = "0";             //reset the current customer's steps value to 0
                curTimeCnt.Text = "0";              //reset the current customer's duration value to 0
            }
            //if there is a customer that is selected from the customer queue
            else if (SelectedCustomers.SelectedIndex >= 0)
            {
                Customers cust = new Customers();           //create a new customer instance
                GDirections dir = new GDirections();        //create a new directions instance
                cust = customerList[SelectedCustomers.SelectedIndex];       //set the customer instance to to the customer index that was selected in the list
                SelectedDestination.Text = cust.Destination.Name;       //set the selected destination value equal to the value of the customer instance's name
                SelectedAdd.Text = cust.custAdd;            //set the selected address value equal to the customer instance's address
                PointLatLng StartLocation = GetLatLngFromAdd(cust.custAdd);     //set the starting location equal to the customer's home address
                PointLatLng EndLocation = cust.Destination.GetCoord();          //set the ending location equal to the customer's destination address
                //create a Google Map directions variable instance with the starting and ending location result
                var x = GMapProviders.GoogleMap.GetDirections(out dir, StartLocation, EndLocation, false, false, false, true, false);
                //if the directions found were valid
                if (x == DirectionsStatusCode.OK)
                {
                    curStepsCnt.Text = dir.Steps.Count.ToString();              //set the current customer's steps count equal to the direction's number of steps
                    curTimeCnt.Text = dir.Duration.Substring(0, 2) + " min";        //set the current customer's duration time to the direction's duration time
                    curMilesCnt.Text = dir.Distance.Substring(0, 3) + " miles";     //set the current customer's mileage to the direction's mileage
                }
            }
        }
        //*******************************************************************************
        //Function timer_Elapsed: A function event that will be performed once the timer
        //in the program has elapsed. The elapsed event handler checks the background worker
        //to see if it is busy, otherwise will run the worker while the program executes
        //********************************************************************************
        void timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            //if the worker is idle and not busy
            if (!worker.IsBusy)
                worker.RunWorkerAsync();        //run the worker while the program is running
        }
        //**************************************************************************************
        //Function worker_DoWork: A function event that will be performed by the background worker's
        //event handler
        //***************************************************************************************
        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            //if all of the customers from the database have not yet been added to the customer
            //queue in the application
            if (AllCustsAdded == false)
            {
                CustomerCount++;                //increment the number of customers that have been added from the database
                //if the amount of customers on the bus is less than the total customers present in the database
                if (CustomerCount < TotalCust)
                {
                    //call the AddCustomerToQueue function which will add the next customer 
                    //in the database to the driver application's customer queue
                    AddCustomerToQueue(databaseCusts[CustomerCount].custName);
                    Random rand = new Random();                     //create a new random variable instance
                    myTimer.Interval = rand.Next(0, maxTime);       //reset the timer interval to a new random value
                    myTimer.Enabled = true;                         //re-enable the timer
                    myTimer.Start();                                //re-run the timer with the new random value
                }
            }
        }
        //**********************************************************************************
        //Function AddCustomerToQueue: Function, which called by the DoWork function, will take a 
        //customer name and add it to the queue while updating certain attribute values in the app.
        //Uses a method invoker if certain objects in the form need to be invoked before changing.
        //**********************************************************************************
        private void AddCustomerToQueue(string CustName)
        {
            MethodInvoker invoker;              //create a new method invoker instance
            //if the customerqueue listbox needs to be invoked
            if (CustomerQueue.InvokeRequired)
            {
                invoker = () => AddCustomerToQueue(CustName);   //set the invoker to this function for the queue
                CustomerQueue.Invoke(invoker);                  //invoke the customer queue
            }
            //if no invoke is required by the customerqueue
            else
            {
                //add the called customer onto the customer queue and setting that he/she needs to be picked up
                CustomerQueue.Items.Add(CustName + " - Needs to be Picked Up");
                //if the customer being added is the only customer in the queue
                if (CustomerQueue.Items.Count == 1)
                    //update the directions to be from the current location to the current customer's address
                    UpdateDirections(CurrentLocation, GetLatLngFromAdd(databaseCusts[CustomerCount].custName));
                //if the total appointments text value on the form needs to be invoked
                if (TotalAppts.InvokeRequired)
                {
                    invoker = () => AddCustomerToQueue(CustName);       //set the invoker to this function for the total appointments value
                    TotalAppts.Invoke(invoker);                         //invoke the appointments value
                }
                //if no invoke is required by the passenger box value
                else
                    TotalAppts.Text = CustomerQueue.Items.Count.ToString();     //display the updated passenger count value
                //if the current customer count added from the database is equal to the total number of customers in the database
                if (CustomerCount == TotalCust)
                    AllCustsAdded = true;       //set the boolean value to alert the application that all customers have been added
            }
        }
    }
    //public class to store all of the customers and their public members that are added from the database
    public class Customers
    {
        public string custName;             //the customer's name (string)
        public string custAdd;              //the customer's address (string)
        public Destinations Destination;    //the customer's chosen destination (Destination)
        public bool HasWheelChair;          //class member to determine whether the customer has a wheelchair (bool)
        public bool PickedUp;               //class membert to determine whether the customer has been picked up (bool)
    }
    //public class to store all of the destinations and their public members that are added from the database
    public class Destinations
    {
        public int ID;                      //the destination's identification number (int)
        public string Name;                 //the destination's identification name (string)
        public double Latitude;             //the destination's latitude coordinate (double)
        public double Longitude;            //the destination's longitude coordinate (double)
        //create a public function to get the coordinates of a location based on the class member's latitude and longitude coordinates
        public PointLatLng GetCoord()
        {
            PointLatLng coord = new PointLatLng();      //create a new latitude/longitude instance
            coord.Lat = Latitude;                       //set that coordinate's latitude equal to the latitude of the destination's lat value
            coord.Lng = Longitude;                      //set that coordinate's longitude equal to the longitude of the destination's long value
            return coord;                   //return that lat/long instance's value
        }
    }
}