﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MySql.Data.MySqlClient;

namespace Door2DoorApp
{
//**********************************************************************************
// UIForm.cs - The Home User Interface Screen of the application where the driver types
// in his/her starting location information, either with HQ's address, a manual address,
// or manual latitude/longitude coordinate information
//**********************************************************************************
    public partial class UIForm : Form
    {
        public UIForm()
        {
            InitializeComponent();
        }
        //if the user presses the quit application button, prompt them if they want to do so and quit the tool
        private void CloseBtn_Click(object sender, EventArgs e)
        {
            DialogResult result1 = MessageBox.Show("Are you sure you would like to exit the application?", "Quit Application", MessageBoxButtons.YesNo);
            if (result1.ToString() == "Yes")
                Environment.Exit(0);
        }
        
        private void BeginBtn_Click(object sender, EventArgs e)
        {
            double num;
            //if the user didn't type in any starting information, prompt them to do so
            if (HQBtn.Checked == false && (LatitudeVal.Text == "" || LongitudeVal.Text == "") && AddressVal.Text == "")
                MessageBox.Show("You did not select any starting location information to use for this form. Please type in valid starting information before continuing.", "No starting location information entered", MessageBoxButtons.OK);
            else if (MaxSeats.Text == "")
                MessageBox.Show("You did not type in any value for the maximum seats. Please ensure that you enter a value greater than zero before continuing.");
            //if the maximum seats or the lat/long was not in a number format, prompt the user that it has to be so
            else if (double.TryParse(MaxSeats.Text, out num) == false)
                MessageBox.Show("The value that you entered for the maximum number of seats was not a number. Please ensure that you enter a value greater than zero before continuing.");
            else if (CoordinatesCheck.Checked == true && ((double.TryParse(LatitudeVal.Text, out num) == false) || (double.TryParse(LongitudeVal.Text, out num))))
                MessageBox.Show("The value that you entered for the latitude or longitude was not a number. Please ensure that you enter a valid number coordinates information before continuing.");
            //if the maximum seats value is less than 1, prompt the user that it has to be greater than 0
            else if (Convert.ToInt16(MaxSeats.Text) < 1)
                MessageBox.Show("The maximum number of seats that you entered is not a valid input. The value must be greater than zero seats.");
            else
            {
                UIForm newForm = this;      //reference the current UI form to a new UIForm variable
                this.Hide();            //hide this form from the screen's view
                //create a new driver application instance and have it appear, reference this current UI form to get access to it's variables
                DriverApp appForm = new DriverApp(ref newForm);
                appForm.Show();
            }
        }
        //if the user clicks on the manual address button, hide the lat/long textboxes and reveal the address textboxes
        private void AddressCheck_CheckedChanged(object sender, EventArgs e)
        {
            if (AddressCheck.Checked == true)
            {
                AddressVal.Visible = true;
                LatitudeVal.Visible = false;
                LongitudeVal.Visible = false;
            }
            else
                AddressVal.Visible = false;
        }
        //if the user clicks on the HQ button, hide the lat/long and the manual address textboxes
        private void HQBtn_CheckedChanged(object sender, EventArgs e)
        {
            if (HQBtn.Checked == true)
            {
                LatitudeVal.Visible = false;
                LongitudeVal.Visible = false;
                AddressVal.Visible = false;
            }
        }
        //if the coordinates button is checked, hide the manual address textbox and reveal the lat/long textboxes
        private void CoordinatesCheck_CheckedChanged(object sender, EventArgs e)
        {
            if (CoordinatesCheck.Checked == true)
            {
                LatitudeVal.Visible = true;
                LongitudeVal.Visible = true;
                AddressVal.Visible = false;
            }
            else
            {
                LatitudeVal.Visible = false;
                LongitudeVal.Visible = false;
            }
        }
        //if the user clicks the X button on the top righthand corner of the screen, prompt them to quit before exiting the application
        private void UI_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.Visible == true)
            {
                DialogResult result1 = MessageBox.Show("Are you sure you would like to exit the application?", "Quit Application", MessageBoxButtons.YesNo);
                if (result1.ToString() == "Yes")
                    Environment.Exit(0);
                else
                    e.Cancel = false;
            }
        }
        //create accessor to get the address value if it was entered in this form
        public string GetStartAddress
        {
            get { return AddressVal.Text; }
        }
        //create accessor to get the latitude value if it was entered in this form
        public string GetStartLatitude
        {
            get { return LatitudeVal.Text; }
        }
        //create accessor to get the longitude value if it was entered in this form
        public string GetStartLongitude
        {
            get { return LongitudeVal.Text; }
        }
        //create an accessor to determine whether the HQ button was checked or not to tell
        //the application whether or not the driver's starting address location is at headquarters
        public bool HQStart
        {
            get { return HQBtn.Checked; }
        }
        //create an accessor to get the maximum seats on the bus if it was entered in this form
        public int GetMaxSeats
        {
            get { return Convert.ToInt16(MaxSeats.Text); }
        }
    }
}